# Syncropated.py 
#  
#  Copyright (c) 2006 INdT (Instituto Nokia de Technologia)
#  
#  Author: Kenneth Rohde Christiansen <kenneth.christiansen@gmail.com>
# 
#  This program is free software; you can redistribute it and/or 
#  modify it under the terms of the GNU General Public License as 
#  published by the Free Software Foundation; either version 2 of the
#  License, or (at your option) any later version.
# 
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
# 
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#  USA

import sys
import os
import statvfs
import gtk
import gobject
import threading
import syncropated.data

from gazpacho.loader.loader import ObjectBuilder

import DeviceDetection
import SongTransfer
import PlaylistParserRhythmbox
import PlaylistParserPLS
import PlaylistParserM3U

import PhotoAlbumReaderPicasa
import PhotoAlbumReaderFSpot

import FreeSpace
import Utils

gtk.gdk.threads_init()

DATA_DIR = os.path.join("/usr/", 'share/syncropated/data/')
UI_PATH = os.path.join("/usr/", 'share/syncropated/data/ui/syncropated-ui.glade')

print DATA_DIR
print UI_PATH

(
	LOCAL_COLUMN_SYNC,
	LOCAL_COLUMN_NAME,
	LOCAL_COLUMN_TYPE,
) = range(3)

(
	REMOTE_COLUMN_PIXBUF,
	REMOTE_COLUMN_NAME,
) = range(2)

(
	CONTENTS_TYPE_PLAYLIST,
	CONTENTS_TYPE_PHOTOALBUM,
) = range(2)

class SynchronizingThread(threading.Thread):

	def set_contents(self, playlists, photoalbums):
		self.playlists = playlists
		self.photoalbums = photoalbums
	
	def run (self):
		global view
		global model
                
		for playlist in self.playlists:
			if view.model.stop_transfer == True:
				break
			SongTransfer.transfer_songs(view.main_window, view, playlist, playlist[0], model.device[1])
			gtk.gdk.threads_enter()
			model.read_remote_contents()
			view.refresh_remote_contents()
			view.refresh_freespace()
			gtk.gdk.threads_leave()

		for photoalbum in self.photoalbums:
			if view.model.stop_transfer == True:
				break			
			SongTransfer.transfer_photoalbums(view.main_window, view, photoalbum, photoalbum[0], model.device[1])
			gtk.gdk.threads_enter()
			model.read_remote_contents()
			view.refresh_remote_contents()
			view.refresh_freespace()
			gtk.gdk.threads_leave()

		gtk.gdk.threads_enter()
		view.main_window.set_sensitive(True)
		view.progress_dialog.hide()
		view.progressbar.set_fraction(0.0)
		view.model.transferred_data = 0
		gtk.gdk.threads_leave()


class SyncropatedNoDevicesFoundError(Exception):
	def __init__(self, value):
		self.value = value
	def __str__(self):
		return `self.value`

class View(object):
	def __init__(self, model):
		super(View, self).__init__()

		self.model = model
		self.model.view = self

		self.widgets = ObjectBuilder (UI_PATH)
		self.widgets.signal_autoconnect(View.__dict__)

		self.main_window = self.widgets.get_widget("main_window")
		self.error_dialog = self.widgets.get_widget("error_dialog")
		self.progress_dialog = self.widgets.get_widget("progress_dialog")
		self.progressbar = self.widgets.get_widget("progressbar")
		self.progress_label = self.widgets.get_widget("progress_label")
		self.progress_image = self.widgets.get_widget("progress_image")
		self.remote_logo_label = self.widgets.get_widget("remote_logo_label")
		self.remote_logo_image = self.widgets.get_widget("remote_logo_image")
		self.local_logo_image = self.widgets.get_widget("local_logo_image")
		self.icon_arrow = self.widgets.get_widget("icon_arrow")
		self.local_treeview = self.widgets.get_widget("treeview_local")	
		self.remote_treeview = self.widgets.get_widget("treeview_remote")
		self.action_vbox = self.widgets.get_widget("action_vbox")


		self.local_model = gtk.TreeStore(bool, str, int)
		self.remote_model = gtk.ListStore(gtk.gdk.Pixbuf, str)

		self.local_treeview.set_model(self.local_model)
		self.remote_treeview.set_model(self.remote_model)

		self.freespace = FreeSpace.FreeSpace(1.0)
		self.freespace.show()
		self.action_vbox.pack_end(self.freespace, expand=False, fill=True, padding=6)

		self.main_window.set_property("icon-name", 'syncropated')

		self.icons = {
	            'Nokia 770': os.path.join(DATA_DIR, 'icons/770.png'),
		}

		## Select Device dialog

		self.select_device_dialog = self.widgets.get_widget("select_device_dialog")
		self.select_device_dialog.set_property("icon-name", 'syncropated')
		self.select_device_treeview = self.widgets.get_widget("select_device_treeview")

		self.device_model = gtk.ListStore(str)
		self.select_device_treeview.set_model(self.device_model)

		## Setup widgets
	
		self.setup_icons()
		self.setup_treeviews()

	# Gives us the ability to do: widgets['widget_name'].action()
	def __getitem__(self, key):
		return self.widgets.get_widget(key)

	def on_about_menuitem_clicked(self):
		dialog = gtk.AboutDialog()
		dialog.set_name("Syncropated!")
		dialog.set_website("http://syncropated.garage.maemo.org")
		dialog.set_comments("Media Synchronizer")
		dialog.set_website_label("Project Website")
		icon = gtk.icon_theme_get_default().load_icon("syncropated", 128, 0)
		dialog.set_logo(icon)
		dialog.set_authors(["Kenneth Rohde Christiansen <kenneth.christiansen@gmail.com>"])
		dialog.set_artists(["Lapo Calamandrei <calamandrei@gmail.com>", "Andreas Nilsson <nisses.mail@home.se>"])
		dialog.set_copyright("Copyright © 2006 INdT")
		dialog.set_property("icon-name", 'syncropated')
		dialog.show()

		dialog.run()
		dialog.destroy()

	def on_stopbutton_clicked(self):
		global model

		model.stop_transfer = True

	def on_syncbutton_clicked(self):
		global model
		global view

		space_required = 0

		iter = view.local_model.get_iter_first()
		counter = 0

		# Filter contents

		new_playlists = []
		new_photoalbums = []
		iter = view.local_model.get_iter_first()

		while (iter != None):
			child_iter = view.local_model.iter_children (iter)
			counter = 0
			while (child_iter != None):
			
				if view.local_model.get_value(child_iter, LOCAL_COLUMN_SYNC) == True:
					contents_type = view.local_model.get_value(child_iter, LOCAL_COLUMN_TYPE)

					if contents_type == CONTENTS_TYPE_PLAYLIST:

						playlist = model.local_playlists[counter]

						try:
							model.remote_playlist_name_hash[playlist[0]]
							dialog = gtk.MessageDialog(view.main_window, 
								gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT, 
								gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO, 
								"Syncropated! has detected that the playlist '%s' is already present " % playlist[0] +
								"on the remote device. Would you like to replace it?")
							dialog.set_title("Replace existing playlist?")
							dialog.show()

							if dialog.run() == gtk.RESPONSE_YES:
								new_playlists += [playlist]
							else:
								print "Ignoring playlist '%s'" % playlist[0]
							dialog.destroy()
						except KeyError, e:			
							new_playlists += [playlist]
					
					if contents_type == CONTENTS_TYPE_PHOTOALBUM:
						photoalbum = model.local_photoalbums[counter]

						try:
							model.remote_photoalbum_name_hash[photoalbum[0]]
							dialog = gtk.MessageDialog(view.main_window, 
								gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT, 
								gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO, 
								"Syncropated! has detected that the photo album '%s' is already present " % photoalbum[0] +
								"on the remote device. Would you like to replace it?")
							dialog.set_title("Replace existing photo album?")
							dialog.show()

							if dialog.run() == gtk.RESPONSE_YES:
								new_photoalbums += [photoalbum]
							else:
								print "Ignoring photo album '%s'" % photoalbum[0]
							dialog.destroy()
						except KeyError, e:			
							new_photoalbums += [photoalbum]

				child_iter = view.local_model.iter_next(child_iter)
				counter += 1

			iter = view.local_model.iter_next(iter)	

		# Calculate space required

		for playlist in new_playlists:
			space_required += playlist[1]

		for photoalbum in new_photoalbums:
			space_required += photoalbum[1]			

		print "Space Required: " + str(space_required)
		print "Space Available: " + str(Utils.freespace(model.device[1]))

		view.model.space_required = space_required

		view.main_window.set_sensitive(False)

		if space_required < Utils.freespace(model.device[1]):
			view.show_progress_meter()
			sync_thread = SynchronizingThread()
			sync_thread.set_contents(new_playlists, new_photoalbums)
			sync_thread.start()
		else:
			dialog = gtk.MessageDialog(view.main_window, 
					gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT, 
					gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE, 
					"The current Mass Storage Device doesn't have enough free space " +
					"in order to transfer all the playlists selected")
			dialog.set_title("Not enough space on device.")
			dialog.show()

			if dialog.run() == gtk.RESPONSE_CLOSE:
				dialog.destroy()
				view.main_window.set_sensitive(True)
	
		return

	def on_mainwindow_destroy(self):
		gtk.main_quit()

	def on_quit_menuitem_clicked(self):
		gtk.main_quit()

	def sync_toggled(self, cell, path, model):
		paths = path.split(":")
		
		if len(paths) == 2:
			iter = model.get_iter((int(paths[0]),int(paths[1])))

			sync = model.get_value(iter, LOCAL_COLUMN_SYNC)
			sync = not sync
			model.set(iter, LOCAL_COLUMN_SYNC, sync)
		else:
			# We have a parent
			iter = model.get_iter((int(paths[0]),))
			child_iter = model.iter_children(iter)

			sync = model.get_value(iter, LOCAL_COLUMN_SYNC)
			sync = not sync
			model.set(iter, LOCAL_COLUMN_SYNC, sync)

			while (child_iter != None):
				model.set_value(child_iter, LOCAL_COLUMN_SYNC, sync)
				child_iter = model.iter_next(child_iter)


	def setup_treeviews(self):

		## Local playlists treeview
		brenderer =  gtk.CellRendererToggle()
		brenderer.connect('toggled', self.sync_toggled, self.local_model)
	 	renderer = gtk.CellRendererText()

		local_column = gtk.TreeViewColumn("Contents")
		local_column.pack_start(brenderer, expand=False)
		local_column.add_attribute(brenderer, "active", LOCAL_COLUMN_SYNC)
		local_column.pack_start(renderer, expand=True)
		local_column.add_attribute(renderer, "markup", LOCAL_COLUMN_NAME)

		self.local_treeview.append_column(local_column)
		self.local_treeview.set_property("show-expanders", True)
		self.local_treeview.expand_all()

		## Remote playlists treeview
		prenderer =  gtk.CellRendererPixbuf()
		remote_column = gtk.TreeViewColumn("Contents")
		remote_column.pack_start(prenderer, expand=False)
		remote_column.add_attribute(prenderer, "pixbuf", REMOTE_COLUMN_PIXBUF)
		remote_column.pack_start(renderer, expand=True)
		remote_column.add_attribute(renderer, "markup", REMOTE_COLUMN_NAME)

		self.remote_treeview.append_column(remote_column)

		## Device selection treeview
		devices_column = gtk.TreeViewColumn("Mass Storage Devices", renderer, text=0)
		self.select_device_treeview.append_column(devices_column)
		self.select_device_treeview.get_selection().set_mode(gtk.SELECTION_SINGLE)

	def setup_icons(self):
		self.remote_logo_image.set_from_pixbuf(gtk.icon_theme_get_default().load_icon("applications-multimedia", 64, 0))
		self.local_logo_image.set_from_pixbuf(gtk.icon_theme_get_default().load_icon("applications-multimedia", 64, 0))
		self.icon_arrow.set_from_pixbuf(gtk.icon_theme_get_default().load_icon("forward", 48, 0))
		self.progress_image.set_from_pixbuf(gtk.icon_theme_get_default().load_icon("sound", 48, 0))

	def refresh_local_contents(self):
		self.local_model.clear()

		parent = self.local_model.append(None, [ False, "<b>Playlists</b>", CONTENTS_TYPE_PLAYLIST ])
		
		for playlist in self.model.local_playlists:
	        	self.local_model.append(parent, [ False, playlist[0], CONTENTS_TYPE_PLAYLIST ])

		parent = self.local_model.append(None, [ False, "<b>Photo Albums</b>", CONTENTS_TYPE_PHOTOALBUM ])

		for album in self.model.local_photoalbums:
			self.local_model.append(parent, [ False, album[0], CONTENTS_TYPE_PHOTOALBUM ])

		self.local_treeview.expand_all()

	def refresh_remote_contents(self):
		self.remote_model.clear()

		if not self.model.remote_playlists and not self.model.remote_photoalbums:
			self.remote_model.append([ None, "No contents available" ])
			return

		if self.model.remote_playlists:
			pixbuf = gtk.icon_theme_get_default().load_icon("sound", 16, 0)
			for playlist in self.model.remote_playlists:
	        		self.remote_model.append([ pixbuf, playlist[0] ])

		if self.model.remote_photoalbums:
			pixbuf = gtk.icon_theme_get_default().load_icon("image", 16, 0)
			for photoalbum in self.model.remote_photoalbums:
	        		self.remote_model.append([ pixbuf, photoalbum[0] ])

	def show_progress_meter(self):
			self.progress_dialog.set_transient_for(self.main_window)
			self.progress_dialog.set_destroy_with_parent(True)
			self.progress_dialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
			self.progressbar.set_fraction(0.0)
			self.progress_dialog.set_title("Synchronizing...")
			self.progressbar.set_text("0%")
			self.progress_dialog.set_property("icon-name", 'syncropated')
			self.progress_dialog.show()

	def refresh_progress_meter(self):
			self.progressbar.set_fraction(((view.model.transferred_data * 100) / view.model.space_required) / 100.0)
			self.progressbar.set_text(str(((view.model.transferred_data * 100) / view.model.space_required)) + "%")

	def refresh_contents(self):
		self.refresh_local_contents()
		self.refresh_remote_contents()

	def refresh_freespace(self):
		if model.device:
			self.freespace.set_percentage(Utils.freespace_percentage(model.device[1]))
		self.freespace.queue_draw()

	def refresh_all(self):
		self.refresh_contents()
		self.refresh_freespace()

	def set_device(self, device):
		if not device:
			self.remote_logo_label.set_label("<b>No device!</b>")
			self.remote_logo_image.set_from_pixbuf(gtk.icon_theme_get_default().load_icon("applications-multimedia", 64, 0))
		else:
			self.remote_logo_label.set_label("<b>My " + device[0] + "</b>")
			if self.icons.get(device[0]) == None:
				self.remote_logo_image.set_from_pixbuf(gtk.icon_theme_get_default().load_icon("applications-multimedia", 64, 0))
			else:
				self.remote_logo_image.set_from_file(self.icons.get(device[0])) 

	def choose_device(self, devices):
		self.main_window.set_sensitive(False)

		for device in devices:
	        	self.device_model.append([ device[0] ])

		self.select_device_dialog.set_title("Select device")
		self.select_device_dialog.show()

		device_no = self.select_device_dialog.run()

		self.main_window.set_sensitive(True)
		self.select_device_dialog.hide()

		return device_no

	def on_select_clicked(self):
		global view

		(model, iter) = view.select_device_treeview.get_selection().get_selected()

		device_no = model.get_path(iter)[0]

		# returns the device_no to the .run method
		view.select_device_dialog.response(device_no)

	def show(self):
		self.main_window.show()


class Model(object):
	def __init__(self):
		self.device = None
		self.view = None
		self.stop_transfer = False

		# Local contents
		self.local_playlists = []
		self.local_photoalbums = []

		# Remote contents
		self.remote_playlists = []
		self.remote_playlist_name_hash = {}
		self.remote_photoalbums = []
		self.remote_photoalbum_name_hash = {}	
		
		self.transferred_data = 0
		self.space_required = 0

	def handle_device(self):
		devices = DeviceDetection.get_compatible_devices()

		if not devices:
			self.view.set_device(None)
			self.read_local_contents()
			raise SyncropatedNoDevicesFoundError("No devices found") 

		if len(devices) > 1:
			self.device = devices[self.view.choose_device(devices)]
		else:
			self.device = devices[0]

		self.view.set_device(self.device)

		self.read_contents()

	def read_contents(self):
		self.read_local_contents()
		self.read_remote_contents()

	def read_local_contents(self):
		self.local_playlists += PlaylistParserRhythmbox.parse_playlists("~/.gnome2/rhythmbox/playlists.xml")
		self.local_playlists += PlaylistParserM3U.parse_playlists("~/.kde/share/apps/amarok/playlists/")
		self.local_photoalbums += PhotoAlbumReaderPicasa.read_albums("~/PicasaDocuments/My Pictures")
		self.local_photoalbums += PhotoAlbumReaderFSpot.read_albums("~/Photos")

	def read_remote_contents(self):
		if self.device:
			self.remote_playlists = PlaylistParserPLS.parse_playlists(self.device[1])
			self.remote_photoalbums = PhotoAlbumReaderPicasa.read_albums(self.device[1] + "/Photos")

			for playlist in self.remote_playlists:
				self.remote_playlist_name_hash[playlist[0]] = playlist

			for photoalbum in self.remote_photoalbums:
				self.remote_photoalbum_name_hash[photoalbum[0]] = photoalbum

		else:
			self.view.remote_logo_label.set_label("<b>No device!</b>")
			raise SyncropatedNoDevicesFoundError("No devices found")

def init_app():
	global model
	global view

	model = Model()
	view = View(model)

	no_devices_found = False
	try:
		model.handle_device()
	except SyncropatedNoDevicesFoundError, e:
		view.main_window.set_sensitive(False)
		no_devices_found = True

	view.refresh_all()
	view.show()

	if no_devices_found:
		dialog = gtk.MessageDialog(view.main_window, 
				gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT, 
				gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE, 
				"Syncropated! was not able to find any compatible Mass Storages Device. "
				+ "\n\nPlease make sure you device is plugged into the computer.")
		dialog.set_property("icon-name", 'syncropated')
		dialog.set_title("No devices found")
		dialog.show()

		if dialog.run() == gtk.RESPONSE_CLOSE:
			sys.exit(1)

	gtk.main()

if __name__ == "__main__":
	init_app()
